#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

#region Global Enums

public enum anaRANDrawMethod {Diamonds, Detail, None}
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Plots diamonds on the chart for a squeeze
    /// </summary>
    [Description("Plots diamonds on the chart for narrow range and inside bars")]
    public class anaRangeAnalysis : Indicator
    {
        #region Variables
       
        	private int 				lookbackIB				= 4;
        	private int 				lookbackNRB				= 7;
			private int 				lookbackOB				= 4;
			private int 				lookbackWRB				= 7;
			private int					lookback				= 0;
			private int					opacityPB				= 3;
			private int					opacityTR				= 2;
			private int					alpha					= 0;
			private double				offset					= 0;
			private anaRANDrawMethod	symbol					= anaRANDrawMethod.Diamonds;
            private bool				showPaintBars			= true;
			private bool 				showInside				= true;
			private bool				showDoubleInside		= true;
			private bool 				showNarrowRange			= true;
			private bool 				showOutside				= true;
			private bool 				showWideRange			= true;
			private bool				showTriangles			= false;
			private bool				candles					= true;
			private bool 				inside		 			= false;
			private bool 				doubleInside			= false;
			private bool 				narrowRange				= false;
			private bool 				outside	 				= false;
			private bool 				wideRange				= false;
			private bool				triangle				= false;
			private bool 				soundAlert 				= false;
			private Color				upcloseBar				= Color.Lime;
			private Color 				downcloseBar			= Color.Red;
			private Color				dojiBar					= Color.PeachPuff;
			private Color 				insideBar				= Color.Goldenrod;	
			private Color 				doubleInsideBar			= Color.Yellow;	
			private Color 				narrowRangeBar 			= Color.Plum;	
			private Color 				narrowInsideBar 		= Color.Fuchsia;	
			private Color 				outsideBar				= Color.DeepSkyBlue;	
			private Color 				wideRangeBar	 		= Color.RoyalBlue;	
			private Color 				wideOutsideBar 			= Color.Cyan;	
			private Color				triangleLBR				= Color.SlateGray;
  			private int					rearmTime				= 30;
			private string				stringIB				="";
			private string				stringNRB				="";
			private string				stringOB				="";
			private string				stringWRB				="";
			private string 				soundFileIB				= "insidebar.wav";
  			private string 				soundFileDIB			= "doubleinsidebar.wav";
 			private string 				soundFileNRB			= "narrowrangebar.wav";
 			private string				soundFileNIB			= "narrowinsidebar.wav";
			private string				soundFileOB				= "outsidebar.wav";
 			private string 				soundFileWRB			= "widerangebar.wav";
 			private string 				soundFileWOB			= "wideoutsidebar.wav";
			private string				soundFileTR				= "threebartriangle.wav";
		
			private IDataSeries			offsetSeries;
			private BoolSeries			isInsideBar;
			private BoolSeries			isDoubleInsideBar;
			private BoolSeries			isNarrowRangeBar;
			private BoolSeries			isOutsideBar;
			private BoolSeries			isWideRangeBar;
			private BoolSeries			isTriangle;
		
      #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Overlay				= true;
			
			isInsideBar			= new BoolSeries(this);
			isDoubleInsideBar	= new BoolSeries(this);
			isNarrowRangeBar	= new BoolSeries(this);
			isOutsideBar		= new BoolSeries(this);
			isWideRangeBar		= new BoolSeries(this);
			isTriangle			= new BoolSeries(this);
		}

		protected override void OnStartUp()
		{
			offsetSeries = ATR(20);
			lookback = 1 + Math.Max( lookbackIB, Math.Max(lookbackNRB, Math.Max(lookbackOB, lookbackWRB)));
			if (ChartControl != null && ChartControl.ChartStyleType == ChartStyleType.CandleStick)
			{
				candles = true;
				alpha = 25*opacityPB;
			}
			else
				candles = false;
			if(lookbackIB < 3)
				stringIB = "";
			else
				stringIB = Convert.ToString(lookbackIB);
			if(lookbackNRB < 3)
				stringNRB = "";
			else
				stringNRB = Convert.ToString(lookbackNRB);
			if(lookbackOB < 3)
				stringOB = "";
			else
				stringOB = Convert.ToString(lookbackOB);
			if(lookbackWRB < 3)
				stringWRB = "";
			else
				stringWRB = Convert.ToString(lookbackWRB);
		}
		
		/// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < lookback)
			{
				isInsideBar.Set(false);
				isDoubleInsideBar.Set(false);
				isNarrowRangeBar.Set(false);
				isOutsideBar.Set(false);
				isWideRangeBar.Set(false);
				return;
			}
			if (FirstTickOfBar)
				offset = 0.5 * offsetSeries[1];
			
			//Testing for inside bars
			if (Historical || CalculateOnBarClose)
			{
				if (Convert.ToInt32(Range()[0]/TickSize) < Convert.ToInt32(Range()[1]/TickSize) && High[0] <= High[1] && Low[0] >= Low[1]) 
				{
					inside = true;
					for (int i = 2; i < lookbackIB; i++)
						if (Convert.ToInt32(Range()[i]/TickSize) < Convert.ToInt32(Range()[0]/TickSize))					
						{
							inside = false;
							break;
						}				
				}
				else
					inside = false;
				isInsideBar.Set(inside);
				if(inside && isInsideBar[1])
					doubleInside = true;
				else
					doubleInside = false;
				isDoubleInsideBar.Set(doubleInside);
			}
			else if (FirstTickOfBar)
			{
				if (Convert.ToInt32(Range()[1]/TickSize) < Convert.ToInt32(Range()[2]/TickSize) && High[1] < High[2] && Low[1] > Low[2]) 
				{
					inside = true;
					for (int i = 2; i < lookbackIB; i++)
						if (Convert.ToInt32(Range()[i+1]/TickSize) < Convert.ToInt32(Range()[1]/TickSize))					
						{
							inside = false;
							break;
						}				
				}
				else
					inside = false;
				isInsideBar.Set(1, inside);
				isInsideBar.Set(false);
				if(inside && isInsideBar[2])
					doubleInside = true;
				else
					doubleInside = false;
				isDoubleInsideBar.Set(1, doubleInside);
				isDoubleInsideBar.Set(false);
			}
				
			//Testing for narrow range bars
			if (Historical || CalculateOnBarClose)
			{
				narrowRange = true;
				for (int i = 1; i < lookbackNRB; i++)
					if (Convert.ToInt32(Range()[i]/TickSize) <= Convert.ToInt32(Range()[0]/TickSize))					
					{
						narrowRange = false;
						break;
					}				
				isNarrowRangeBar.Set(narrowRange);
			}
			else if (FirstTickOfBar)
			{
				narrowRange = true;
				for (int i = 1; i < lookbackNRB; i++)
					if (Convert.ToInt32(Range()[i+1]/TickSize) <= Convert.ToInt32(Range()[1]/TickSize))					
					{
						narrowRange = false;
						break;
					}				
				isNarrowRangeBar.Set(1, narrowRange);
				isNarrowRangeBar.Set(false);
			}				
			
			// Testing for outside bars
			if(High[0] > High[1] && Low[0] < Low[1])
			{
				outside = true;
				for (int i = 2; i < lookbackOB; i++)
					if (Convert.ToInt32(Range()[i]/TickSize) > Convert.ToInt32(Range()[0]/TickSize))					
					{
						outside = false;
						break;
					}	
			}
			else 
				outside = false;
			isOutsideBar.Set(outside);
			
			// Testing for wide ranging bars
			wideRange = true;
			for (int i = 1; i < lookbackWRB; i++)
				if(Convert.ToInt32(Range()[i]/TickSize) >= Convert.ToInt32(Range()[0]/TickSize))
					wideRange = false;
			isWideRangeBar.Set(wideRange);
			//Testing for triangles
			if (Historical || CalculateOnBarClose)
			{
				if ((High[0] < High[1] || High[0] < High[2]) && (Low[0] > Low[1] || Low[0] > Low[2])) 
					triangle = true;
				else
					triangle = false;
				isTriangle.Set(triangle);
			}
			else if (FirstTickOfBar)
			{
				if ((High[1] < High[2] || High[1] < High[3]) && (Low[1] > Low[2] || Low[1] > Low[3])) 
					triangle = true;
				else
					triangle = false;
				isTriangle.Set(1, triangle);
				isTriangle.Set(false);
			}
				
			
			// Drawing diamonds or text above the price bars
			if(Historical || CalculateOnBarClose)
			{	
				if(symbol == anaRANDrawMethod.Diamonds)
				{
					if (showNarrowRange && narrowRange && !(showInside && inside))
						DrawDiamond("NR"+CurrentBar, true, 0, High[0] + 0.5*offset, narrowRangeBar);
					else if (showDoubleInside && doubleInside)
						DrawDiamond("NR"+CurrentBar, true, 0, High[0] + 0.5*offset, doubleInsideBar);
					else if (!(showNarrowRange && narrowRange) && showInside && inside)
						DrawDiamond("NR"+CurrentBar, true, 0, High[0] + 0.5*offset, insideBar);
					else if (showNarrowRange && narrowRange && showInside && inside)
						DrawDiamond("NR"+CurrentBar, true, 0, High[0] + 0.5*offset, narrowInsideBar);
					if (showTriangles && triangle)
						DrawTriangleUp("TR"+CurrentBar, true, 0, Low[0] - 0.5*offset, triangleLBR);
				}
				else if(symbol == anaRANDrawMethod.Detail)
				{
					if (showNarrowRange && narrowRange && !(showInside && inside))
						DrawText("NR"+CurrentBar, "NR" + stringNRB, 0, High[0] + 0.5*offset, narrowRangeBar); 
					else if (showDoubleInside && doubleInside)
						DrawText("NR"+CurrentBar,"DI" + stringIB, 0, High[0] + 0.5*offset, doubleInsideBar);
					else if (!(showNarrowRange && narrowRange) && showInside && inside)
						DrawText("NR"+CurrentBar, "IB" + stringIB, 0, High[0] + 0.5*offset, insideBar); 
					else if (showNarrowRange && narrowRange && showInside && inside)
						DrawText("NR"+CurrentBar, "IB" + stringNRB, 0, High[0] + 0.5*offset, narrowInsideBar);
					if(showTriangles && triangle)
						DrawTriangle("TR"+CurrentBar, false, 0, Close[0], 2, Math.Max(High[1],High[2]), 2, Math.Min(Low[1], Low[2]), triangleLBR, triangleLBR, opacityTR);	
				}
			}
			else if(FirstTickOfBar)
			{
				if(symbol == anaRANDrawMethod.Diamonds)
				{
					if (showNarrowRange && narrowRange && !(showInside && inside))
						DrawDiamond("NR"+CurrentBar, true, 1, High[1] + 0.5*offset, narrowRangeBar);
					else if (showDoubleInside && doubleInside)
						DrawDiamond("NR"+CurrentBar, true, 1, High[1] + 0.5*offset, doubleInsideBar);
					else if (!(showNarrowRange && narrowRange) && showInside && inside)
						DrawDiamond("NR"+CurrentBar, true, 1, High[1] + 0.5*offset, insideBar);
					else if (showNarrowRange && narrowRange && showInside && inside)
						DrawDiamond("NR"+CurrentBar, true, 1, High[1] + 0.5*offset, narrowInsideBar);
					if (showTriangles && triangle)
						DrawTriangleUp("TR"+CurrentBar, true, 1, Low[1] - 0.5*offset, triangleLBR);
				}
				else if(symbol == anaRANDrawMethod.Detail)
				{
					if (showNarrowRange && narrowRange && !(showInside && inside))
						DrawText("NR"+CurrentBar, "NR" + stringNRB, 1, High[1] + 0.5*offset, narrowRangeBar); 
					else if (showDoubleInside && doubleInside)
						DrawText("NR"+CurrentBar,"DI" + stringIB, 1, High[1] + 0.5*offset, doubleInsideBar);
					else if (!(showNarrowRange && narrowRange) && showInside && inside)
						DrawText("NR"+CurrentBar, "IB" + stringIB, 1, High[1] + 0.5*offset, insideBar); 
					else if (showNarrowRange && narrowRange && showInside && inside)
						DrawText("NR"+CurrentBar, "IB" + stringNRB, 1, High[1] + 0.5*offset, narrowInsideBar);
					if(showTriangles && triangle)
						DrawTriangle("TR"+CurrentBar, false, 0, Close[1], 3, Math.Max(High[2],High[3]), 3, Math.Min(Low[2], Low[3]), triangleLBR, triangleLBR, opacityTR);	
				}
			}
			
			if(symbol == anaRANDrawMethod.Diamonds)
			{
				if (showWideRange && wideRange && !(showOutside && outside))
					DrawDiamond("WR"+CurrentBar,true, 0, High[0] + 0.5*offset, wideRangeBar);
				else if (!(showWideRange && wideRange) && showOutside && outside)
					DrawDiamond("WR"+CurrentBar,true, 0, High[0] + 0.5*offset, outsideBar);
				else if (showWideRange && wideRange && showOutside && outside)
					DrawDiamond("WR"+CurrentBar,true, 0, High[0] + 0.5*offset, wideOutsideBar);
			}
			else if(symbol == anaRANDrawMethod.Detail)
			{
				if (showWideRange && wideRange && !(showOutside && outside))
					DrawText("WR"+CurrentBar, "WR" + stringWRB, 0, High[0] + 0.5*offset, wideRangeBar); 
				else if (!(showWideRange && wideRange) && showOutside && outside)
					DrawText("WR"+CurrentBar, "OB" + stringOB, 0, High[0] + 0.5*offset, outsideBar); 
				else if (showWideRange && wideRange && showOutside && outside)
					DrawText("WR"+CurrentBar, "OB" + stringWRB, 0, High[0] + 0.5*offset, wideOutsideBar);
			}
			
			if(showPaintBars)
			{
				if(Historical || CalculateOnBarClose)
				{	
					if(Close[0] > Open[0])
					{
						BarColorSeries[0] = upcloseBar;
						CandleOutlineColorSeries[0] = upcloseBar;
					}
					else if (Close[0] < Open[0])
					{
						BarColorSeries[0] = downcloseBar;
						CandleOutlineColorSeries[0] = downcloseBar;
					}
					else
					{
						BarColorSeries[0] = dojiBar;
						CandleOutlineColorSeries[0] = dojiBar;
					}
					if (showNarrowRange && narrowRange && !(showInside && inside))
					{
						BarColorSeries[0] = narrowRangeBar;
						CandleOutlineColorSeries[0] = narrowRangeBar;
					}
					else if (showDoubleInside && doubleInside)
					{
						BarColorSeries[0] = doubleInsideBar;
						CandleOutlineColorSeries[0] = doubleInsideBar;
					}
					else if (!(showNarrowRange && narrowRange) && showInside && inside)
					{
						BarColorSeries[0] = insideBar;
						CandleOutlineColorSeries[0] = insideBar;
					}
					else if (showNarrowRange && narrowRange && showInside && inside)
					{
						BarColorSeries[0] = narrowInsideBar;
						CandleOutlineColorSeries[0] = narrowInsideBar;
					}
					else if (showTriangles && triangle)
					{
						BarColorSeries[0] = triangleLBR;
						CandleOutlineColorSeries[0] = triangleLBR;
					}
					if (showWideRange && wideRange && !(showOutside && outside))
					{
						BarColorSeries[0] = wideRangeBar;
						CandleOutlineColorSeries[0] = wideRangeBar;
					}
					else if (!(showWideRange && wideRange) && showOutside && outside)
					{
						BarColorSeries[0] = outsideBar;
						CandleOutlineColorSeries[0] = outsideBar;
					}
					else if (showWideRange && wideRange && showOutside && outside)
					{
						BarColorSeries[0] = wideOutsideBar;
						CandleOutlineColorSeries[0] = wideOutsideBar;
					}
					if(candles)
					{
						if(Open[0] < Close[0]) 
							BarColorSeries[0] = Color.FromArgb(alpha, CandleOutlineColorSeries[0]);
					}				
				}
				else 
				{	
					if(FirstTickOfBar)
					{
						if (showNarrowRange && narrowRange && !(showInside && inside))
						{
							BarColorSeries[1] = narrowRangeBar;
							CandleOutlineColorSeries[1] = narrowRangeBar;
						}
						else if (showDoubleInside && doubleInside)
						{
							BarColorSeries[1] = doubleInsideBar;
							CandleOutlineColorSeries[1] = doubleInsideBar;
						}
						else if (!(showNarrowRange && narrowRange) && showInside && inside)
						{
							BarColorSeries[1] = insideBar;
							CandleOutlineColorSeries[1] = insideBar;
						}
						else if (showNarrowRange && narrowRange && showInside && inside)
						{
							BarColorSeries[1] = narrowInsideBar;
							CandleOutlineColorSeries[1] = narrowInsideBar;
						}
						else if (showTriangles && triangle)
						{
							BarColorSeries[1] = triangleLBR;
							CandleOutlineColorSeries[1] = triangleLBR;
						}
						if(candles)
						{
							if(Open[1] < Close[1]) 
								BarColorSeries[1] = Color.FromArgb(alpha, CandleOutlineColorSeries[1]);
						}						
					}
					if(Close[0] > Open[0])
					{
						BarColorSeries[0] = upcloseBar;
						CandleOutlineColorSeries[0] = upcloseBar;
					}
					else if (Close[0] < Open[0])
					{
						BarColorSeries[0] = downcloseBar;
						CandleOutlineColorSeries[0] = downcloseBar;
					}
					else
					{
						BarColorSeries[0] = dojiBar;
						CandleOutlineColorSeries[0] = dojiBar;
					}				
					if (showWideRange && wideRange && !(showOutside && outside))
					{
						BarColorSeries[0] = wideRangeBar;
						CandleOutlineColorSeries[0] = wideRangeBar;
					}
					else if (!(showWideRange && wideRange) && showOutside && outside)
					{
						BarColorSeries[0] = outsideBar;
						CandleOutlineColorSeries[0] = outsideBar;
					}
					else if (showWideRange && wideRange && showOutside && outside)
					{
						BarColorSeries[0] = wideOutsideBar;
						CandleOutlineColorSeries[0] = wideOutsideBar;
					}					
					if(candles)
					{
						if(Open[0] < Close[0]) 
							BarColorSeries[0] = Color.FromArgb(alpha, CandleOutlineColorSeries[0]);
					}		
				}			
			}
			
			if (soundAlert && !Historical && IsConnected())
			{
				if(CalculateOnBarClose || FirstTickOfBar)
				{	
					if (showTriangles && triangle)
					{	
						try
						{
								Alert("TriangleLBR", NinjaTrader.Cbi.Priority.Medium,"Triangle LBR", soundFileTR, rearmTime, Color.Black, triangleLBR);
						}
						catch{}
					}
					if (showNarrowRange && narrowRange && !(showInside && inside))
					{	
						try
						{
								Alert("NarrowRangeBar", NinjaTrader.Cbi.Priority.Medium,"Narrow Range Bar", soundFileNRB, rearmTime, Color.Black, narrowRangeBar);
						}
						catch{}
					}
					else if (showDoubleInside && doubleInside)
					{	
						try
						{
								Alert("DoubleInsideBar", NinjaTrader.Cbi.Priority.Medium,"Double Inside Bar", soundFileDIB, rearmTime, Color.Black, doubleInsideBar);
						}
						catch{}
					}
					else if (!(showNarrowRange && narrowRange) && showInside && inside)
					{	
						try
						{
								Alert("InsideBar", NinjaTrader.Cbi.Priority.Medium,"Inside Bar", soundFileIB, rearmTime, Color.Black, insideBar);
						}
						catch{}
					}
					else if (showNarrowRange && narrowRange && showInside && inside)
					{	
						try
						{
								Alert("NarrowInsideBar", NinjaTrader.Cbi.Priority.Medium,"Narrow Inside Bar", soundFileNIB, rearmTime, Color.Black, narrowInsideBar);
						}
						catch{}
					}
				}
				if (showWideRange && wideRange && !(showOutside && outside))
				{	
					try
					{
							Alert("WideRangeBar", NinjaTrader.Cbi.Priority.Medium,"Wide Range Bar", soundFileWRB, rearmTime, Color.Black, wideRangeBar);
					}
					catch{}
				}
				else if (!(showWideRange && wideRange) && showOutside && outside)
				{	
					try
					{
							Alert("OutsideBar", NinjaTrader.Cbi.Priority.Medium,"Outside Bar", soundFileOB, rearmTime, Color.Black, outsideBar);
					}
					catch{}
				}
				else if (showWideRange && wideRange && showOutside && outside)
				{	
					try
					{
							Alert("WideOutsideBar", NinjaTrader.Cbi.Priority.Medium,"Wide Outside Bar", soundFileWOB, rearmTime, Color.Black, wideOutsideBar);
					}
					catch{}
				}
			}
        }

        #region Properties
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public BoolSeries IsInsideBar
		{
			get { return isInsideBar; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public BoolSeries IsDoubleInsideBar
		{
			get { return isDoubleInsideBar; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public BoolSeries IsNarrowRangeBar
		{
			get { return isNarrowRangeBar; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public BoolSeries IsOutsideBar
		{
			get { return isOutsideBar; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public BoolSeries IsWideRangeBar
		{
			get { return isWideRangeBar; }
		}
		
		[Description("Lookback for Inside Narrow Range Bars")]
        [GridCategory("Parameters")]
  		[Gui.Design.DisplayName("Lookback inside bars")]
     	public int LookbackIB
        {
            get { return lookbackIB; }
            set { lookbackIB = Math.Max(2, value); }
        }

     	[Description("Lookback for Narrow Range Bars")]
        [GridCategory("Parameters")]
  		[Gui.Design.DisplayName("Lookback narrow range bars")]
     	public int LookbackNRB
        {
            get { return lookbackNRB; }
            set { lookbackNRB = Math.Max(2, value); }
        }

     	[Description("Lookback for Outside Wide Range Bars")]
        [GridCategory("Parameters")]
  		[Gui.Design.DisplayName("Lookback outside bars")]
     	public int LookbackOB
        {
            get { return lookbackOB; }
            set { lookbackOB = Math.Max(2, value); }
        }

     	[Description("Lookback for Wide Range Bars")]
        [GridCategory("Parameters")]
  		[Gui.Design.DisplayName("Lookback wide range bars")]
     	public int LookbackWRB
        {
            get { return lookbackWRB; }
            set { lookbackWRB = Math.Max(2, value); }
        }

		/// <summary>
		/// </summary>
		[Description("Select Drawing Method for Hidden DataSeries")]
		[Category("Display Options")]
		[Gui.Design.DisplayNameAttribute("Drawing method")]
		public anaRANDrawMethod Symbol
		{
			get { return symbol; }
			set { symbol = value; }
		}
		
		[Description("Set to true to display inside bars")]
        [Category("Display Options")]
		[Gui.Design.DisplayName("Display inside bars")]
        public bool ShowInside
        {
            get { return showInside; }
            set { showInside = value; }
        }
		
		[Description("Set to true to display double inside bars")]
        [Category("Display Options")]
		[Gui.Design.DisplayName("Display double inside bars")]
        public bool ShowDoubleInside
        {
            get { return showDoubleInside; }
            set { showDoubleInside = value; }
        }
		
		[Description("Set to true to display narrow range bars")]
        [Category("Display Options")]
		[Gui.Design.DisplayName("Display narrow range bars")]
        public bool ShowNarrowRange
        {
            get { return showNarrowRange; }
            set { showNarrowRange = value; }
        }
		
		[Description("Set to true to display outside bars")]
        [Category("Display Options")]
		[Gui.Design.DisplayName("Display outside bars")]
        public bool ShowOutside
        {
            get { return showOutside; }
            set { showOutside = value; }
        }
		
		[Description("Set to true to display wide range bars")]
        [Category("Display Options")]
		[Gui.Design.DisplayName("Display wide range bars")]
        public bool ShowWideRange
        {
            get { return showWideRange; }
            set { showWideRange = value; }
        }

		[Description("Set to true to display three bar triangles as described by Linda Bradford Raschke")]
        [Category("Display Options")]
		[Gui.Design.DisplayName("Display three bar triangles")]
        public bool ShowTriangles
        {
            get { return showTriangles; }
            set { showTriangles = value; }
        }

		[Description("Set to true to display paint bars")]
        [Category("Display Options")]
		[Gui.Design.DisplayName("Show paint bars")]
        public bool ShowPaintBars
        {
            get { return showPaintBars; }
            set { showPaintBars = value; }
        }

		[Description("Select opacity for triangles, when triangles are activated")]
        [Category("Display Options")]
  		[Gui.Design.DisplayName("Triangle opacity")]
     	public int OpacityTR
        {
            get { return opacityTR; }
            set { opacityTR = value; }
        }
		
		[Description("Select opacity for upclosing candles, when paintbars are activated")]
        [Category("Display Options")]
  		[Gui.Design.DisplayName("Paintbar opacity")]
     	public int OpacityPB
        {
            get { return opacityPB; }
            set { opacityPB = value; }
        }
		
		/// <summary>
		/// </summary>
		[Description("Color for Upclose Bar")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Upclose bar")]
		public Color UpcloseBar
		{
			get { return upcloseBar; }
			set { upcloseBar = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string UpcloseBarSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(upcloseBar); }
			set { upcloseBar = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Color for downclose bar")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Downclose Bar")]
		public Color DowncloseBar
		{
			get { return downcloseBar; }
			set { downcloseBar = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string DowncloseBarSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(downcloseBar); }
			set { downcloseBar = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Color for doji bar")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Doji bar")]
		public Color DojiBar
		{
			get { return dojiBar; }
			set { dojiBar = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string DojiBarSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(dojiBar); }
			set { dojiBar = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Color for inside bar")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Inside bar")]
		public Color InsideBar
		{
			get { return insideBar; }
			set { insideBar = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string InsideBarSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(insideBar); }
			set { insideBar = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Color for double inside bar")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Double inside bar")]
		public Color DoubleInsideBar
		{
			get { return doubleInsideBar; }
			set { doubleInsideBar = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string DoubleInsideBarSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(doubleInsideBar); }
			set { doubleInsideBar = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Color for narrow range bar")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Narrow range bar")]
		public Color NarrowRangeBar
		{
			get { return narrowRangeBar; }
			set { narrowRangeBar = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string NarrowRangeBarSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(narrowRangeBar); }
			set { narrowRangeBar = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Color for narrow range inside bar")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Narrow range inside bar")]
		public Color NarrowInsideBar
		{
			get { return narrowInsideBar; }
			set { narrowInsideBar = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string NarrowInsideBarSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(narrowInsideBar); }
			set { narrowInsideBar = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("Color for outside bar")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Outside bar")]
		public Color OutsideBar
		{
			get { return outsideBar; }
			set { outsideBar = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string OutsideBarSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(outsideBar); }
			set { outsideBar = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Color for wide range bar")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Wide range bar")]
		public Color WideRangeBar
		{
			get { return wideRangeBar; }
			set { wideRangeBar = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string WideRangeBarSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(wideRangeBar); }
			set { wideRangeBar = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Color for wide range outside bar")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Wide range outside bar")]
		public Color WideRangeOutsideBar
		{
			get { return wideOutsideBar; }
			set { wideOutsideBar = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string WideRangeOutsideBarSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(wideOutsideBar); }
			set { wideOutsideBar = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Color for three bar triangles")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Three bar triangles")]
		public Color TriangleLBR
		{
			get { return triangleLBR; }
			set { triangleLBR = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string TriangleLBRSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(triangleLBR); }
			set { triangleLBR = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[Description("Set to true to activate sounds")]
        [Category("Sound Alerts")]
		[Gui.Design.DisplayName("Active sounds")]
        public bool SoundAlert
        {
            get { return soundAlert; }
            set { soundAlert = value; }
        }
		
		[Description("Sound File for inside bars")]
		[Category("Sound Alerts")]
		[Gui.Design.DisplayNameAttribute("Inside bar")]
		public string SoundFileIB
		{
		get { return soundFileIB; }
		set { soundFileIB = value; }
		}

 		[Description("Sound File for double inside bars")]
		[Category("Sound Alerts")]
		[Gui.Design.DisplayNameAttribute("Double inside bar")]
		public string SoundFileDIB
		{
		get { return soundFileDIB; }
		set { soundFileDIB = value; }
		}

		[Description("Sound File for narrow range bars")]
		[Category("Sound Alerts")]
		[Gui.Design.DisplayNameAttribute("Narrow range bar")]
		public string SoundFileNRB
		{
		get { return soundFileNRB; }
		set { soundFileNRB = value; }
		}

		[Description("Sound File for narrow range inside bars")]
		[Category("Sound Alerts")]
		[Gui.Design.DisplayNameAttribute("Narrow range inside bar")]
		public string SoundFileNIB
		{
		get { return soundFileNIB; }
		set { soundFileNIB = value; }
		}

		[Description("Sound File for outside bars")]
		[Category("Sound Alerts")]
		[Gui.Design.DisplayNameAttribute("Outside bar")]
		public string SoundFileOB
		{
		get { return soundFileOB; }
		set { soundFileOB = value; }
		}

		[Description("Sound file for wide range bars")]
		[Category("Sound Alerts")]
		[Gui.Design.DisplayNameAttribute("Wide range bar")]
		public string SoundFileWRB
		{
		get { return soundFileWRB; }
		set { soundFileWRB = value; }
		}

		[Description("Sound file for wide range outside bars")]
		[Category("Sound Alerts")]
		[Gui.Design.DisplayNameAttribute("Wide range outside bar")]
		public string SoundFileWOB
		{
		get { return soundFileWOB; }
		set { soundFileWOB = value; }
		}
		
		[Description("Rearm time for alert in seconds")]
		[Category("Sound Alerts")]
		[Gui.Design.DisplayNameAttribute("Rearm time (sec)")]
		public int RearmTime
		{
		get { return rearmTime; }
		set { rearmTime = value; }
		}
		#endregion
		
		#region Miscellaneous
		private bool IsConnected()
        {
			if ( Bars.MarketData != null
					&& Bars.MarketData.Connection.PriceStatus == Cbi.ConnectionStatus.Connected
					&& Bars.Session.InSession(Now, Bars.Period, true))
				return true;
			else
            	return false;
        }		
		
		private DateTime Now
		{
			get 
			{ 
				DateTime now = (Bars.MarketData.Connection.Options.Provider == Cbi.Provider.Replay ? Bars.MarketData.Connection.Now : DateTime.Now); 

				if (now.Millisecond > 0)
					now = Cbi.Globals.MinDate.AddSeconds((long) System.Math.Floor(now.Subtract(Cbi.Globals.MinDate).TotalSeconds));

				return now;
			}
		}		
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private anaRangeAnalysis[] cacheanaRangeAnalysis = null;

        private static anaRangeAnalysis checkanaRangeAnalysis = new anaRangeAnalysis();

        /// <summary>
        /// Plots diamonds on the chart for narrow range and inside bars
        /// </summary>
        /// <returns></returns>
        public anaRangeAnalysis anaRangeAnalysis(int lookbackIB, int lookbackNRB, int lookbackOB, int lookbackWRB)
        {
            return anaRangeAnalysis(Input, lookbackIB, lookbackNRB, lookbackOB, lookbackWRB);
        }

        /// <summary>
        /// Plots diamonds on the chart for narrow range and inside bars
        /// </summary>
        /// <returns></returns>
        public anaRangeAnalysis anaRangeAnalysis(Data.IDataSeries input, int lookbackIB, int lookbackNRB, int lookbackOB, int lookbackWRB)
        {
            if (cacheanaRangeAnalysis != null)
                for (int idx = 0; idx < cacheanaRangeAnalysis.Length; idx++)
                    if (cacheanaRangeAnalysis[idx].LookbackIB == lookbackIB && cacheanaRangeAnalysis[idx].LookbackNRB == lookbackNRB && cacheanaRangeAnalysis[idx].LookbackOB == lookbackOB && cacheanaRangeAnalysis[idx].LookbackWRB == lookbackWRB && cacheanaRangeAnalysis[idx].EqualsInput(input))
                        return cacheanaRangeAnalysis[idx];

            lock (checkanaRangeAnalysis)
            {
                checkanaRangeAnalysis.LookbackIB = lookbackIB;
                lookbackIB = checkanaRangeAnalysis.LookbackIB;
                checkanaRangeAnalysis.LookbackNRB = lookbackNRB;
                lookbackNRB = checkanaRangeAnalysis.LookbackNRB;
                checkanaRangeAnalysis.LookbackOB = lookbackOB;
                lookbackOB = checkanaRangeAnalysis.LookbackOB;
                checkanaRangeAnalysis.LookbackWRB = lookbackWRB;
                lookbackWRB = checkanaRangeAnalysis.LookbackWRB;

                if (cacheanaRangeAnalysis != null)
                    for (int idx = 0; idx < cacheanaRangeAnalysis.Length; idx++)
                        if (cacheanaRangeAnalysis[idx].LookbackIB == lookbackIB && cacheanaRangeAnalysis[idx].LookbackNRB == lookbackNRB && cacheanaRangeAnalysis[idx].LookbackOB == lookbackOB && cacheanaRangeAnalysis[idx].LookbackWRB == lookbackWRB && cacheanaRangeAnalysis[idx].EqualsInput(input))
                            return cacheanaRangeAnalysis[idx];

                anaRangeAnalysis indicator = new anaRangeAnalysis();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.LookbackIB = lookbackIB;
                indicator.LookbackNRB = lookbackNRB;
                indicator.LookbackOB = lookbackOB;
                indicator.LookbackWRB = lookbackWRB;
                Indicators.Add(indicator);
                indicator.SetUp();

                anaRangeAnalysis[] tmp = new anaRangeAnalysis[cacheanaRangeAnalysis == null ? 1 : cacheanaRangeAnalysis.Length + 1];
                if (cacheanaRangeAnalysis != null)
                    cacheanaRangeAnalysis.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheanaRangeAnalysis = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Plots diamonds on the chart for narrow range and inside bars
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaRangeAnalysis anaRangeAnalysis(int lookbackIB, int lookbackNRB, int lookbackOB, int lookbackWRB)
        {
            return _indicator.anaRangeAnalysis(Input, lookbackIB, lookbackNRB, lookbackOB, lookbackWRB);
        }

        /// <summary>
        /// Plots diamonds on the chart for narrow range and inside bars
        /// </summary>
        /// <returns></returns>
        public Indicator.anaRangeAnalysis anaRangeAnalysis(Data.IDataSeries input, int lookbackIB, int lookbackNRB, int lookbackOB, int lookbackWRB)
        {
            return _indicator.anaRangeAnalysis(input, lookbackIB, lookbackNRB, lookbackOB, lookbackWRB);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Plots diamonds on the chart for narrow range and inside bars
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaRangeAnalysis anaRangeAnalysis(int lookbackIB, int lookbackNRB, int lookbackOB, int lookbackWRB)
        {
            return _indicator.anaRangeAnalysis(Input, lookbackIB, lookbackNRB, lookbackOB, lookbackWRB);
        }

        /// <summary>
        /// Plots diamonds on the chart for narrow range and inside bars
        /// </summary>
        /// <returns></returns>
        public Indicator.anaRangeAnalysis anaRangeAnalysis(Data.IDataSeries input, int lookbackIB, int lookbackNRB, int lookbackOB, int lookbackWRB)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.anaRangeAnalysis(input, lookbackIB, lookbackNRB, lookbackOB, lookbackWRB);
        }
    }
}
#endregion
